@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Restrict Senders Plugin
-  * Copyright (c) 2004-2008 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2003-2011 Paul Lesniewski <paul@squirrelmail.org>
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -20,10 +20,27 @@
 function restrict_senders_init()
 {
 
-   if (!@include_once (SM_PATH . 'plugins/restrict_senders/data/config.php'))
-      return FALSE;
+   return load_config('restrict_senders',
+                      array('../../config/config_restrict_senders.php', 'data/config.php'),
+                      TRUE, TRUE);
 
-   return TRUE;
+}
+
+
+
+/**
+  * Returns a list of preference names that should not
+  * be controllable by the end user
+  *
+  * @return string A comma-delimited list of preference
+  *                names that the end-user should not
+  *                have control over.
+  *
+  */
+function rs_sensitive_preference_names_do()
+{
+
+   return 'USER_IS_POSSIBLE_SPAMMER, mass_email_send_times, todays_email_count, ';
 
 }
 
@@ -178,7 +195,7 @@ function display_error_on_compose_screen_do()
 
       // error when message had too many recipients 
       //
-      global $restrict_senders_error_too_many_recipients, $color, $report_abuse_too_many_recipients_message;
+      global $restrict_senders_error_too_many_recipients, $color;
       if (sqgetGlobalVar('restrict_senders_error_too_many_recipients', $restrict_senders_error_too_many_recipients, SQ_GET)
        && $restrict_senders_error_too_many_recipients)
       {
@@ -219,22 +236,6 @@ function display_error_on_compose_screen_do()
             else
                plain_error_message($msg, $color);
 
-            if (!empty($report_abuse_too_many_recipients_message))
-               $msg = _($report_abuse_too_many_recipients_message);
-            else
-               $msg = _("NOTICE: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send an email with more than their allotted recipients: ###NUMBER_RECIPIENTS###\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nBODY: ###BODY###");
-               //$msg = _("NOTICE: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send an email with more than their allotted recipients: ###NUMBER_RECIPIENTS###");
-
-            $msg = str_replace(array('###USERNAME###', '###DOMAIN###', 
-                                     '###NUMBER_RECIPIENTS###', '###TO###', 
-                                     '###CC###', '###BCC###', '###BODY###'),
-                               array($username, $domain, $restrictNumberOfRecipients, 
-                                     $send_to, $send_to_cc, $send_to_bcc, $body),
-                               $msg);
-
-
-            rs_report_abuse($msg);
-
             sq_change_text_domain('squirrelmail');
             return;
          }
@@ -245,8 +246,7 @@ function display_error_on_compose_screen_do()
 
       // error when too many emails sent in one day
       //
-      global $restrict_senders_error_too_many_emails_today, 
-             $color, $report_abuse_too_many_today_message;
+      global $restrict_senders_error_too_many_emails_today, $color;
       if (sqgetGlobalVar('restrict_senders_error_too_many_emails_today', $restrict_senders_error_too_many_emails_today, SQ_GET)
        && $restrict_senders_error_too_many_emails_today)
       {
@@ -287,22 +287,6 @@ function display_error_on_compose_screen_do()
             else
                plain_error_message($msg, $color);
 
-            if (!empty($report_abuse_too_many_today_message))
-               $msg = _($report_abuse_too_many_today_message);
-            else
-               $msg = _("WARNING: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send more than their allotted number of emails in one day: ###NUMBER_MAILS_TODAY###\n\nTo allow this user to send mail again today, remove the \"todays_email_count\" setting from their preferences.\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nBODY: ###BODY###");
-               //$msg = _("WARNING: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send more than their allotted number of emails in one day: ###NUMBER_RECIPIENTS###");
-
-            $msg = str_replace(array('###USERNAME###', '###DOMAIN###', 
-                                     '###NUMBER_MAILS_TODAY###', '###TO###', 
-                                     '###CC###', '###BCC###', '###BODY###'),
-                               array($username, $domain, $restrictNumberOfEmailsPerDay, 
-                                     $send_to, $send_to_cc, $send_to_bcc, $body),
-                               $msg);
-
-
-            rs_report_abuse($msg);
-
             sq_change_text_domain('squirrelmail');
             return;
          }
@@ -399,7 +383,12 @@ function check_restrict_senders_do(&$argv)
           $vlogin_restrictSubsequentMassEmailsWhitelist, $rs_debug,
           $whitelisted_addresses, $rs_delimiter, $log_too_many_recipients,
           $log_too_many_today, $log_too_many_subsequent_mass_emails,
-          $log_no_recipients;
+          $log_no_recipients, $rs_clear_prefs_cache_on_send, $draft;
+
+
+   // don't restrict saving of drafts
+   //
+   if (!empty($draft)) return;
 
 
    if (check_sm_version(1, 5, 2))
@@ -433,6 +422,19 @@ function check_restrict_senders_do(&$argv)
    $now = time();
 
 
+   // clear preferences cache to defeat spammers logged
+   // in from more than one location/browser at once
+   //
+   if ($rs_clear_prefs_cache_on_send)
+   {
+      global $prefs_cache, $prefs_are_cached;
+      $prefs_cache = FALSE;
+      $prefs_are_cached = FALSE;
+      sqsession_unregister('prefs_are_cached');
+      sqsession_unregister('prefs_cache');
+   }
+
+
    // grab TO, CC, and BCC fields from message
    //
    $toDomains = array();
@@ -505,7 +507,9 @@ function check_restrict_senders_do(&$argv)
             if (sizeof($newTimes) >= $number_mass_mails)
             {
 
-               global $send_to, $send_to_cc, $send_to_bcc, $subject, $body;
+               global $send_to, $send_to_cc, $send_to_bcc, $subject, $body,
+                      $report_abuse_banned_too_many_subsequent_mass_mails;
+
 
                setPref($data_dir, $username, 'USER_IS_POSSIBLE_SPAMMER', 1);
                $sentDates = '';
@@ -513,6 +517,17 @@ function check_restrict_senders_do(&$argv)
                   $sentDates .= date('H:i:s Y-m-d', $time) . "\n";
 
 
+               $replyToList = array();
+               $fromList = array();
+               $delimiter = '@';
+               $address_delim = ', ';
+
+               foreach ($message->rfc822_header->reply_to as $sender)
+                  $replyToList[] = $sender->mailbox . $delimiter . $sender->host;
+               foreach ($message->rfc822_header->from as $sender)
+                  $fromList[] = $sender->mailbox . $delimiter . $sender->host;
+
+
 //FIXME: could move the logging and sending of alert down below after we let the whitelist-only messages go through...??  but maybe we want to know anyway??
                // log this event - note that to use this,
                // you'll need to have the Squirrel Logger plugin
@@ -528,8 +543,26 @@ function check_restrict_senders_do(&$argv)
 
                // send alert to admin
                //
+               if (empty($report_abuse_banned_too_many_subsequent_mass_mails))
+                  //$message_details = "TO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###\nBODY: ###BODY###";
+                  $message_details = "TO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###\n";
+               else
+                  $message_details = $report_abuse_banned_too_many_subsequent_mass_mails;
+
+               $message_details = str_replace(array('###USERNAME###', '###DOMAIN###', 
+                                                    '###TO###', '###CC###',
+                                                    '###BCC###', '###BODY###',
+                                                    '###FROM###', '###REPLY_TO###',
+                                                    '###SUBJECT###'),
+                                              array($username, $domain, $send_to,
+                                                    $send_to_cc, $send_to_bcc, $body,
+                                                    implode($address_delim, $fromList),
+                                                    implode($address_delim, $replyToList),
+                                                    $subject),
+                                              $message_details);
+
                sq_change_text_domain('restrict_senders');
-               rs_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has sent %d mass emails of %d or more recipients in the last %d minutes.\n\nTimes:\n%s\n\n%s has been BANNED.\n\nTo un-ban this user, remove the USER_IS_POSSIBLE_SPAMMER setting from their preferences.\n\nTO: %s\nCC: %s\nBCC: %s\nBODY: %s"), $username, $domain, sizeof($newTimes), $mass_email_recips, $number_minutes, $sentDates, $username, $send_to, $send_to_cc, $send_to_bcc, $body), sprintf(_(" --- BANNED - %s"), $username));
+               rs_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has sent %d mass emails of %d or more recipients in the last %d minutes.\n\nTimes:\n%s\n\n%s has been BANNED.\n\nTo un-ban this user, remove the USER_IS_POSSIBLE_SPAMMER setting from their preferences.\n\n%s"), $username, $domain, sizeof($newTimes), $mass_email_recips, $number_minutes, $sentDates, $username, $message_details), sprintf(_(" --- BANNED - %s"), $username));
                sq_change_text_domain('squirrelmail');
 
             }
@@ -595,6 +628,21 @@ function check_restrict_senders_do(&$argv)
       && $restrictNumberOfRecipients < sizeof($toDomains) + sizeof($ccDomains) + sizeof($bccDomains))
    {
 
+      global $color, $send_to, $send_to_cc, $send_to_bcc, $subject, $body,
+             $variable_sent_folder, $PHP_SELF,
+             $report_abuse_too_many_recipients_message;
+
+      $replyToList = array();
+      $fromList = array();
+      $delimiter = '@';
+      $address_delim = ', ';
+
+      foreach ($message->rfc822_header->reply_to as $sender)
+         $replyToList[] = $sender->mailbox . $delimiter . $sender->host;
+      foreach ($message->rfc822_header->from as $sender)
+         $fromList[] = $sender->mailbox . $delimiter . $sender->host;
+
+
       // log this event - note that to use this,
       // you'll need to have the Squirrel Logger plugin
       // installed and activated and you'll have to add
@@ -607,8 +655,31 @@ function check_restrict_senders_do(&$argv)
       }
 
 
-      global $color, $send_to, $send_to_cc, $send_to_bcc, $subject, $body,
-             $variable_sent_folder, $PHP_SELF;
+      // send report if needed
+      //
+      if (!empty($report_abuse_too_many_recipients_message))
+         $msg = _($report_abuse_too_many_recipients_message);
+      else
+         $msg = _("NOTICE: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send an email with more than their allotted recipients: ###NUMBER_RECIPIENTS###\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###");
+         //$msg = _("NOTICE: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send an email with more than their allotted recipients: ###NUMBER_RECIPIENTS###\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###\nBODY: ###BODY###");
+         //$msg = _("NOTICE: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send an email with more than their allotted recipients: ###NUMBER_RECIPIENTS###");
+
+      $msg = str_replace(array('###USERNAME###', '###DOMAIN###', 
+                               '###NUMBER_RECIPIENTS###', '###TO###', 
+                               '###CC###', '###BCC###', '###BODY###',
+                               '###FROM###', '###REPLY_TO###', '###SUBJECT###'),
+                         array($username, $domain, $restrictNumberOfRecipients, 
+                               $send_to, $send_to_cc, $send_to_bcc, $body,
+                               implode($address_delim, $fromList),
+                               implode($address_delim, $replyToList),
+                               $subject),
+                         $msg);
+
+      rs_report_abuse($msg);
+
+
+      // now redirect/abort send
+      //
       sqgetGlobalVar('variable_sent_folder', $variable_sent_folder);
 
       if (strpos($PHP_SELF, '?') === FALSE)
@@ -689,7 +760,46 @@ function check_restrict_senders_do(&$argv)
 
 
          global $color, $send_to, $send_to_cc, $send_to_bcc, $subject, $body,
-                $variable_sent_folder, $PHP_SELF;
+                $variable_sent_folder, $PHP_SELF,
+                $report_abuse_too_many_today_message;
+
+
+         $replyToList = array();
+         $fromList = array();
+         $delimiter = '@';
+         $address_delim = ', ';
+
+         foreach ($message->rfc822_header->reply_to as $sender)
+            $replyToList[] = $sender->mailbox . $delimiter . $sender->host;
+         foreach ($message->rfc822_header->from as $sender)
+            $fromList[] = $sender->mailbox . $delimiter . $sender->host;
+
+
+         // send report if needed
+         //
+         if (!empty($report_abuse_too_many_today_message))
+            $msg = _($report_abuse_too_many_today_message);
+         else
+            $msg = _("WARNING: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send more than their allotted number of emails in one day: ###NUMBER_MAILS_TODAY###\n\nTo allow this user to send mail again today, remove the \"todays_email_count\" setting from their preferences.\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###");
+            //$msg = _("WARNING: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send more than their allotted number of emails in one day: ###NUMBER_MAILS_TODAY###\n\nTo allow this user to send mail again today, remove the \"todays_email_count\" setting from their preferences.\n\nTO: ###TO###\nCC: ###CC###\nBCC: ###BCC###\nFROM: ###FROM###\nREPLY-TO: ###REPLY_TO###\nSUBJECT: ###SUBJECT###\nBODY: ###BODY###");
+            //$msg = _("WARNING: User \"###USERNAME###\" (domain \"###DOMAIN###\") has attempted to send more than their allotted number of emails in one day: ###NUMBER_RECIPIENTS###");
+
+         $msg = str_replace(array('###USERNAME###', '###DOMAIN###', 
+                                  '###NUMBER_MAILS_TODAY###', '###TO###', 
+                                  '###CC###', '###BCC###', '###BODY###',
+                                  '###FROM###', '###REPLY_TO###', '###SUBJECT###'),
+                            array($username, $domain, $restrictNumberOfEmailsPerDay, 
+                                  $send_to, $send_to_cc, $send_to_bcc, $body,
+                                  implode($address_delim, $fromList),
+                                  implode($address_delim, $replyToList),
+                                  $subject),
+                            $msg);
+
+         rs_report_abuse($msg);
+
+
+         // now redirect/abort send
+         //
          sqgetGlobalVar('variable_sent_folder', $variable_sent_folder);
 
          if (strpos($PHP_SELF, '?') === FALSE)
@@ -1005,6 +1115,9 @@ function rs_buildQueryString($varName, $queryName = '')
   *                            to be parsed.  (OPTIONAL; default
   *                            is to use the file called 
   *                            send_restrictions.php in the 
+  *                            SquirrelMail config directory, or
+  *                            if that file is not found, the file
+  *                            called send_restrictions.php in the
   *                            restrict_senders/data directory)
   * @param boolean $configtest When TRUE, this function will test
   *                            whether or not the sender rules
@@ -1030,6 +1143,8 @@ function parseSenderRules($rules_file='', $configtest=FALSE)
 
 
    if (empty($rules_file))
+      $rules_file = SM_PATH . 'config/send_restrictions.php';
+   if (!file_exists($rules_file))
       $rules_file = SM_PATH . 'plugins/restrict_senders/data/send_restrictions.php';
 
 
@@ -1460,9 +1575,9 @@ function filterAddresses($toDomains, $ccDomains, $bccDomains,
 /**
   * Reports abuse to administrator
   *
-  * @param $error string Error text indicating what the abuse problem is
-  * @param $title string Short text for inclusion in email subject line
-  * @param $message object The message object that is being attempted to 
+  * @param string $error   Error text indicating what the abuse problem is
+  * @param string $title   Short text for inclusion in email subject line
+  * @param object $message The message object that is being attempted to 
   *                        be sent (optional, ignored if not given)
   *
   */
@@ -1480,7 +1595,8 @@ function rs_report_abuse($error, $title='', $message='')
           $rs_smtp_sitewide_pass, $useSendmail, $smtpServerAddress,
           $smtpPort, $sendmail_path, $sendmail_args, $pop_before_smtp,
           $encode_header_key, $smtp_auth_mech, $smtp_sitewide_user,
-          $smtp_sitewide_pass;
+          $smtp_sitewide_pass,
+          $include_server_env_with_abuse_report;
 
 
    // get global variable for versions of PHP < 4.1
@@ -1519,14 +1635,17 @@ function rs_report_abuse($error, $title='', $message='')
       ob_end_clean();
    }
 
-   $body .= "\n\n=========================================================\n"
-         . _("OFFENDING SERVER ENVIRONMENT")
-         . "\n=========================================================\n\n";
-   ob_start();
-   global $_SERVER;
-   print_r($_SERVER);
-   $body .= ob_get_contents();
-   ob_end_clean();
+   if ($include_server_env_with_abuse_report)
+   {
+      $body .= "\n\n=========================================================\n"
+            . _("OFFENDING SERVER ENVIRONMENT")
+            . "\n=========================================================\n\n";
+      ob_start();
+      global $_SERVER;
+      print_r($_SERVER);
+      $body .= ob_get_contents();
+      ob_end_clean();
+   }
 
 
    $subject = _("[WEBMAIL ABUSE]")
